# STA 410/2102, Assignment #1, Spring 2014 - Functions for finding MLE.


# Find the log probability of a single observation given the model parameters.
#
# Arguments:
#              count   Count of how many children were born this day
#              time    The number of the day (integer from 1 up)
#              params  Vector of model parameters, in order a, b, w
#
# Returns:     The log of the probability of the count at the given time,
#              using the given parameters, accounting for all possibilities
#              for the number of twins.

logp <- function (count, time, params)
{ 
    lambda <- abs(params[1] + params[2] * time)
    twin_prob <- 1 / (1 + exp(-params[3]))
    pr <- 0
    for (twins in 0 : (count %/% 2)) {
        pr <- pr + dpois (twins, lambda * twin_prob) * 
                   dpois (count - 2*twins, lambda * (1-twin_prob))
    }
    log(pr)
}


# Compute the log likelihood for parameters given a vector of observations, y.
#
# Arguments:
#              y       Vector of counts of children born each day
#              params  Vector of model parameters, in order a, b, w
#
# Returns:     The log of the probability of the vector of observed counts.

logl <- function (y, params)
{
    ll <- 0
    for (i in 1:length(y)) {
        ll <- ll + logp (y[i], i, params)
    }
    ll
}


# Find the Maximum Likelihood Estimates for parameters given observations.
#
# Arguments:
#              y        Vector of counts of children born each day
#              initial  Initial value to use when optimizing the parameters 
#              method   Optimization method to use, either "nlm" or "optim"
#
# Returns:     A list with the elements "estimate" for the MLE parameter
#              estimate, "se" for the standard errors of these estimated
#              parameters, and "logl" for the log likelihood at the MLE.

mle <- function (y, initial, method)
{
    if (method == "nlm") {
        res <- nlm (function (params) -logl(y,params), initial, hessian = TRUE)
        if (res$code > 3)
            warning("possible non-convergence")
        val <- list (logl = -res$minimum, estimate = res$estimate)
    }

    else if (method == "optim") {
        res <- optim(initial, function (params) -logl(y,params), hessian = TRUE)
        if (res$convergence > 0)
            warning("possible non-convergence")
        val <- list (logl = -res$value, estimate = res$par)
    }

    else 
        stop("invalid method")

    val$se <- sqrt(diag(solve(res$hessian)))
    names(val$estimate) <- names(val$se) <- c("a","b","w")
    val
}
